@@ -1,16 +1,21 @@
 /* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- *
- * Copyright 1998-2006 Decklin Foster <decklin@red-bean.com>. This
- * program is free software; please see LICENSE for details. */
+ * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
+ * This program is free software; see LICENSE for details. */
 
 #ifndef AEWM_CLIENTS_AECLIENTS_H
 #define AEWM_CLIENTS_AECLIENTS_H
 
 #include <stdlib.h>
 #include "common.h"
+#include "atom.h"
 
 #define DEF_RC  "/etc/X11/aewm/clientsrc"
-#define SPACING 3
+#define SPACING 2
+
+enum {
+    LAUNCH,
+    SWITCH
+};
 
 typedef void *menu_t;
 
