@@ -1,14 +1,14 @@
 /* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- *
- * Copyright 1998-2006 Decklin Foster <decklin@red-bean.com>. This
- * program is free software; please see LICENSE for details. */
+ * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
+ * This program is free software; see LICENSE for details. */
 
 #ifndef AEWM_H
 #define AEWM_H
 
-#define VERSION "1.2.7"
+#define VERSION "1.3.0"
 
 #include "common.h"
+#include "strut.h"
 #include <stdlib.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -39,8 +39,9 @@
 #define DEF_NEW2 "aemenu --switch"
 #define DEF_NEW3 "xterm"
 
-#define DEF_BW   1
-#define DEF_PAD  3
+#define DEF_BW    1
+#define DEF_PAD   3
+#define DEF_DESKS 4
 
 /* A few useful masks made up out of X's basic ones. `ChildMask' is a
  * silly name, but oh well. */
@@ -70,6 +71,11 @@
 #define BW(c) (opt_bw)
 #endif
 
+/* Hooray for magic numbers */
+
+#define DESK_ALL 0xFFFFFFFF
+#define ON_CUR_DESK(c) ((c)->desktop == cur_desk || (c)->desktop == DESK_ALL)
+
 /* Multipliers for calling gravitate */
 
 enum {
@@ -117,21 +123,25 @@ enum {
 typedef struct client client_t;
 
 struct client {
-    client_t   *next;
-    char       *name;
-    XSizeHints *size;
-    Window     window, frame, trans;
-    Colormap   cmap;
-    int        x, y, width, height;
-    int        ignore_unmap;
+    client_t    *next;
+    char        *name;
+    XSizeHints  *size;
+    Window      window, frame, trans;
+    Colormap    cmap;
+    int         x, y, width, height;
+    int         save_x, save_y, save_width, save_height;
+    int         ignore_unmap;
+    long        desktop;
+    Bool        shaded;
+    Bool        max_vert;
+    Bool        max_horz;
 #ifdef SHAPE
-    Bool       has_been_shaped;
-#endif
-#ifdef MWM_HINTS
-    Bool       has_title, has_border;
+    Bool        has_been_shaped;
 #endif
+    Bool        has_title;
+    Bool        has_border;
 #ifdef XFT
-    XftDraw    *xftdraw;
+    XftDraw     *xftdraw;
 #endif
 };
 
@@ -143,6 +153,7 @@ extern Display *dpy;
 extern Window root;
 extern int screen;
 extern client_t *head_client;
+extern long cur_desk;
 extern XFontStruct *font;
 #ifdef XFT
 extern XftFont *xftfont;
@@ -160,17 +171,27 @@ extern Atom wm_state;
 extern Atom wm_change_state;
 extern Atom wm_protos;
 extern Atom wm_delete;
-#ifdef GNOME_PDA
-extern Atom gnome_pda;
-#endif
 #ifdef MWM_HINTS
 extern Atom mwm_hints;
 #endif
+extern Atom net_current_desktop;
+extern Atom net_num_desktops;
+extern Atom net_client_list;
+extern Atom net_wm_desktop;
+extern Atom net_wm_state;
+extern Atom net_wm_state_shaded;
+extern Atom net_wm_state_max_vert;
+extern Atom net_wm_state_max_horz;
+extern Atom net_wm_strut;
+extern Atom net_wm_strut_partial;
+extern Atom net_wm_window_type;
+extern Atom net_wm_window_type_dock;
 extern char *opt_new1;
 extern char *opt_new2;
 extern char *opt_new3;
 extern int opt_bw;
 extern int opt_pad;
+extern int opt_ndesks;
 #ifdef SHAPE
 extern Bool shape;
 extern int shape_event;
@@ -182,8 +203,9 @@ extern void do_event_loop(void);
 /* client.c */
 extern client_t *find_client(Window, int);
 extern int theight(client_t *);
-extern void set_wm_state(client_t *, int);
-extern long get_wm_state(client_t *);
+extern int get_wm_state(client_t *);
+extern int set_wm_state(client_t *, int);
+extern void handle_net_wm_state_item(unsigned char *, void *);
 extern void send_config(client_t *);
 extern void remove_client(client_t *, int);
 extern void redraw(client_t *);
@@ -196,10 +218,15 @@ extern void set_shape(client_t *);
 extern void make_new_client(Window);
 
 /* manage.c */
+extern void switch_desk(int);
+extern void desk_update(client_t *);
 extern void move(client_t *);
 extern void resize(client_t *);
-extern void hide(client_t *);
+extern void iconify_win(client_t *);
 extern void send_wm_delete(client_t *);
+extern void shade_win(client_t *);
+extern void maximize_win(client_t *);
+extern void toggle_all_desks(client_t *);
 
 /* misc.c */
 extern void err(const char *, ...);
@@ -209,6 +236,8 @@ extern int handle_xerror(Display *, XErrorEvent *);
 extern int ignore_xerror(Display *, XErrorEvent *);
 extern int send_xmessage(Window, Atom, long);
 extern void get_mouse_position(int *, int *);
+extern void remove_from_atom(Window, Atom, Atom, long);
+extern void accumulate_struts(int, strut_t *);
 #ifdef DEBUG
 extern void show_event(XEvent);
 extern void dump_title(client_t *, const char *, char);
