@@ -1,10 +1,10 @@
 /* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- *
- * Copyright 1998-2006 Decklin Foster <decklin@red-bean.com>. This
- * program is free software; please see LICENSE for details. */
+ * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
+ * This program is free software; see LICENSE for details. */
 
 #include <X11/Xmd.h>
 #include "aewm.h"
+#include "atom.h"
 
 client_t *find_client(Window w, int mode)
 {
@@ -13,7 +13,7 @@ client_t *find_client(Window w, int mode)
     if (mode == MATCH_FRAME) {
         for (c = head_client; c; c = c->next)
             if (c->frame == w) return c;
-    } else if (mode == MATCH_WINDOW) {
+    } else /* mode == MATCH_WINDOW */ {
         for (c = head_client; c; c = c->next)
             if (c->window == w) return c;
     }
@@ -27,56 +27,63 @@ client_t *find_client(Window w, int mode)
 
 int theight(client_t *c)
 {
-    int font_height;
-
     if (!c) return 0;
 
 #ifdef MWM_HINTS
     if (!c->has_title) return 0;
 #endif
-
 #ifdef XFT
-    font_height = xftfont->ascent + xftfont->descent;
+    return (c->trans ? 0 : xftfont->ascent + xftfont->descent) +
+        2*opt_pad + BW(c);
 #else
-    font_height = font->ascent + font->descent;
+    return (c->trans ? 0 : font->ascent + font->descent) +
+        2*opt_pad + BW(c);
 #endif
-    return c->trans ? font_height / 2 : font_height + 2*opt_pad + BW(c);
-}
-
-/* Attempt to follow the ICCCM by explicity specifying 32 bits for
- * this property. Does this goof up on 64 bit systems? */
-
-void set_wm_state(client_t *c, int state)
-{
-    CARD32 data[2];
-
-    data[0] = state;
-    data[1] = None; /* Icon? We don't need no steenking icon. */
-
-    XChangeProperty(dpy, c->window, wm_state, wm_state,
-        32, PropModeReplace, (unsigned char *)data, 2);
 }
 
 /* If we can't find a WM_STATE we're going to have to assume
  * Withdrawn. This is not exactly optimal, since we can't really
  * distinguish between the case where no WM has run yet and when the
  * state was explicitly removed (clients are allowed to either set the
- * atom to Withdrawn or just remove it... yuck.) */
+ * atom to Withdrawn or just remove it... yuck.)
+ *
+ * WM_STATE actually has a second element, the old ICCCM-style icon,
+ * but we don't care about it so we can just use get_atom here. */
 
-long get_wm_state(client_t *c)
+int get_wm_state(client_t *c)
 {
-    Atom real_type; int real_format;
-    long state = WithdrawnState;
-    unsigned long items_read, bytes_left;
-    unsigned char *data;
+    long state;
 
-    if (XGetWindowProperty(dpy, c->window, wm_state, 0L, 2L, False,
-            wm_state, &real_type, &real_format, &items_read, &bytes_left,
-            &data) == Success && items_read) {
-        state = *(long *)data;
-        XFree(data);
+    if (get_atom(c->window, wm_state, wm_state, &state))
+        return state;
+    else
+        return WithdrawnState;
+}
+
+int set_wm_state(client_t *c, int state)
+{
+    return set_atom(c->window, wm_state, wm_state, state);
+}
+
+/* This can be called before we map, so check c->frame. If c->frame is
+ * still None, we merely set the variables, and then later (in
+ * reparent) adjust window and/or frame sizes based on them. */
+
+void handle_net_wm_state_item(unsigned char *data, void *cb_data)
+{
+    Atom prop = *(Atom *)data;
+    client_t *c = cb_data;
+
+    if (prop == net_wm_state_shaded && !c->shaded) {
+        if (c->frame) shade_win(c);
+        else c->shaded = 1;
+    } else if (prop == net_wm_state_max_vert && !c->max_vert) {
+        c->max_vert = 1;
+        if (c->frame && c->max_vert && c->max_horz) maximize_win(c);
+    } else if (prop == net_wm_state_max_horz && !c->max_horz) {
+        c->max_horz = 1;
+        if (c->frame && c->max_vert && c->max_horz) maximize_win(c);
     }
-    return state;
 }
 
 /* This will need to be called whenever we update our client_t stuff.
@@ -123,8 +130,21 @@ void remove_client(client_t *c, int mode)
     dump_removal(c, mode);
 #endif
 
-    if (mode == REMOVE_WITHDRAW) set_wm_state(c, WithdrawnState);
-    else if (mode == REMOVE_REMAP) XMapWindow(dpy, c->window);
+    if (mode == REMOVE_WITHDRAW) {
+        set_wm_state(c, WithdrawnState);
+    } else /* mode == REMOVE_REMAP */ {
+        if (c->max_vert && c->max_horz) {
+            c->x = c->save_x;
+            c->y = c->save_y;
+            c->width = c->save_width;
+            c->height = c->save_height;
+            XResizeWindow(dpy, c->window,
+                c->width, c->height);
+        }
+        XMapWindow(dpy, c->window);
+    }
+
+    remove_from_atom(root, net_client_list, XA_WINDOW, c->window);
 
     gravitate(c, GRAV_UNDO);
     XReparentWindow(dpy, c->window, root, c->x, c->y);
@@ -160,17 +180,17 @@ void remove_client(client_t *c, int mode)
 
 void redraw(client_t *c)
 {
+    int innerbox_x = 0;
+    int innerbox_y = 0;
+    int innerbox_w = 0;
+    int innerbox_h = 0;
+
 #ifdef MWM_HINTS
     if (!c->has_title) return;
 #endif
 
     XClearWindow(dpy, c->frame);
-    XDrawLine(dpy, c->frame, border_gc,
-        0, theight(c) - BW(c) + BW(c)/2,
-        c->width, theight(c) - BW(c) + BW(c)/2);
-    XDrawLine(dpy, c->frame, border_gc,
-        c->width - theight(c) + BW(c)/2, 0,
-        c->width - theight(c) + BW(c)/2, theight(c));
+
     if (!c->trans && c->name) {
 #ifdef XFT
         XftDrawString8(c->xftdraw, &xft_fg,
@@ -182,6 +202,43 @@ void redraw(client_t *c)
             c->name, strlen(c->name));
 #endif
     }
+
+    XDrawLine(dpy, c->frame, border_gc,
+        0, theight(c) - BW(c) + BW(c)/2,
+        c->width, theight(c) - BW(c) + BW(c)/2);
+    XDrawLine(dpy, c->frame, border_gc,
+        c->width - theight(c) + BW(c)/2, 0,
+        c->width - theight(c) + BW(c)/2, theight(c));
+    XDrawLine(dpy, c->frame, border_gc,
+        c->width - 2*theight(c) + BW(c)/2, 0,
+        c->width - 2*theight(c) + BW(c)/2, theight(c));
+
+    /* Ugh. What a mess. Don't ask me how I figured these numbers out.
+     * I hate graphics programming. */
+
+    if ((c->max_vert && c->max_horz) || c->desktop == DESK_ALL) {
+        innerbox_x = c->width - 2*theight(c) + BW(c) + (theight(c)-BW(c))/4;
+        innerbox_y = (theight(c)-BW(c))/4;
+        innerbox_w = (theight(c)-BW(c)) - (theight(c)-BW(c))/4*2 - BW(c)%2;
+        innerbox_h = (theight(c)-BW(c)) - (theight(c)-BW(c))/4*2 - BW(c)%2;
+    }
+
+    if (c->max_vert && c->max_horz) {
+        XDrawRectangle(dpy, c->frame, border_gc,
+            innerbox_x, innerbox_y, innerbox_w, innerbox_h);
+    }
+    if (c->desktop == DESK_ALL) {
+        XDrawLine(dpy, c->frame, border_gc,
+            innerbox_x + BW(c)%2,
+            innerbox_y + BW(c)%2,
+            innerbox_x + innerbox_w,
+            innerbox_y + innerbox_h);
+        XDrawLine(dpy, c->frame, border_gc,
+            innerbox_x + BW(c)%2,
+            innerbox_y + innerbox_h - 1,
+            innerbox_x + innerbox_w,
+            innerbox_y - (BW(c)+1)%2);
+    }
 }
 
 /* Window gravity is a mess to explain, but we don't need to do much
@@ -207,7 +264,7 @@ void gravitate(client_t *c, int mode)
 
     if (mode == GRAV_APPLY)
         c->y += dy;
-    else if (mode == GRAV_UNDO)
+    else /* mode == GRAV_UNDO */
         c->y -= dy;
 }
 
