@@ -1,15 +1,13 @@
 /* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- *
- * Copyright 1998-2006 Decklin Foster <decklin@red-bean.com>. This
- * program is free software; please see LICENSE for details. */
+ * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
+ * This program is free software; see LICENSE for details. */
 
-#include <X11/Xatom.h>
 #include "aewm.h"
+#include "atom.h"
 
 static void handle_button_press(XButtonEvent *);
 static void handle_button_release(XButtonEvent *);
 static void handle_configure_request(XConfigureRequestEvent *);
-static void handle_circulate_request(XCirculateRequestEvent *);
 static void handle_map_request(XMapRequestEvent *);
 static void handle_unmap_event(XUnmapEvent *);
 static void handle_destroy_event(XDestroyWindowEvent *);
@@ -43,8 +41,6 @@ void do_event_loop(void)
                 handle_button_release(&ev.xbutton); break;
             case ConfigureRequest:
                 handle_configure_request(&ev.xconfigurerequest); break;
-            case CirculateRequest:
-                handle_circulate_request(&ev.xcirculaterequest); break;
             case MapRequest:
                 handle_map_request(&ev.xmaprequest); break;
             case UnmapNotify:
@@ -88,19 +84,32 @@ void do_event_loop(void)
 static void handle_button_press(XButtonEvent *e)
 {
     client_t *c = find_client(e->window, MATCH_FRAME);
-    int in_box;
+    int in_box1, in_box2;
 
     if (e->window == root) {
         root_button_pressed = 1;
     } else if (c) {
-        in_box = (e->x >= c->width - theight(c)) && (e->y <= theight(c));
-        switch (e->button) {
-            case Button1:
-                in_box ? hide(c) : raise_win(c); break;
-            case Button2:
-                in_box ? resize(c) : move(c); break;
-            case Button3:
-                in_box ? send_wm_delete(c) : lower_win(c); break;
+        in_box1 = (e->x >= c->width - theight(c)) && (e->y <= theight(c));
+        in_box2 = (e->x >= c->width - 2*theight(c) - BW(c)) &&
+            (e->x < c->width - theight(c) - BW(c)) && (e->y <= theight(c));
+        if (in_box1) {
+            switch (e->button) {
+                case Button1: iconify_win(c); break;
+                case Button2: resize(c); break;
+                case Button3: send_wm_delete(c); break;
+            }
+        } else if (in_box2) {
+            switch (e->button) {
+                case Button1: shade_win(c); break;
+                case Button2: maximize_win(c); break;
+                case Button3: toggle_all_desks(c); break;
+            }
+        } else {
+            switch (e->button) {
+                case Button1: raise_win(c); break;
+                case Button2: move(c); break;
+                case Button3: lower_win(c); break;
+            }
         }
     }
 }
@@ -176,20 +185,6 @@ static void handle_configure_request(XConfigureRequestEvent *e)
     XConfigureWindow(dpy, e->window, e->value_mask, &wc);
 }
 
-/* The only window that we will circulate children for is the root
- * (because nothing else would make sense). After a client requests that
- * the root's children be circulated, the server will determine which
- * window needs to be raised or lowered, and so all we have to do is
- * make it so. */
-
-static void handle_circulate_request(XCirculateRequestEvent *e)
-{
-    if (e->parent == root) {
-        if (e->place == PlaceOnBottom) XLowerWindow(dpy, e->window);
-        else /* e->place == PlaceOnTop */ XRaiseWindow(dpy, e->window);
-    }
-}
-
 /* Two possiblilies if a client is asking to be mapped. One is that
  * it's a new window, so we handle that if it isn't in our clients
  * list anywhere. The other is that it already exists and wants to
@@ -246,22 +241,29 @@ static void handle_destroy_event(XDestroyWindowEvent *e)
 
 /* If a client wants to iconify itself (boo! hiss!) it must send a
  * special kind of ClientMessage. We might set up other handlers here
- * but there's nothing else required by the ICCCM. */
+ * but there's nothing else required by the ICCCM. The EWMH spec,
+ * OTOH, defines a few messages to be sent to the root window. */
 
 static void handle_client_message(XClientMessageEvent *e)
 {
     client_t *c = find_client(e->window, MATCH_WINDOW);
 
-    if (c && e->message_type == wm_change_state &&
-        e->format == 32 && e->data.l[0] == IconicState) hide(c);
+    if (e->window == root) {
+        if (e->message_type == net_current_desktop &&
+                e->format == 32)
+            switch_desk(e->data.l[0]);
+    } else if (c) {
+        if (e->message_type == wm_change_state &&
+                e->format == 32 && e->data.l[0] == IconicState)
+            iconify_win(c);
+    }
 }
 
-/* All that we have cached is the name and the size hints, so we only
- * have to check for those here. A change in the name means we have to
- * immediately wipe out the old name and redraw size hints only get
- * used when we need them. Note that the actual redraw call both
- * clears and draws; before Xft, the XClearWindow call was only made
- * in this function. */
+/* We only need to check for the stuff we have cached here. A change
+ * in the name means we have to immediately wipe out the old name and
+ * redraw size hints only get used when we need them. Note that the
+ * actual redraw call both clears and draws; before Xft, the
+ * XClearWindow call was only made in this function. */
 
 static void handle_property_change(XPropertyEvent *e)
 {
@@ -270,15 +272,18 @@ static void handle_property_change(XPropertyEvent *e)
 
     if (!c) return;
 
-    switch (e->atom) {
-        case XA_WM_NAME:
-            if (c->name) XFree(c->name);
-            XFetchName(dpy, c->window, &c->name);
-            redraw(c);
-            break;
-        case XA_WM_NORMAL_HINTS:
-            XGetWMNormalHints(dpy, c->window, c->size, &dummy);
-            break;
+    if (e->atom == XA_WM_NAME) {
+        if (c->name) XFree(c->name);
+        XFetchName(dpy, c->window, &c->name);
+        redraw(c);
+    } else if (e->atom == XA_WM_NORMAL_HINTS) {
+        XGetWMNormalHints(dpy, c->window, c->size, &dummy);
+    } else if (e->atom == net_wm_state) {
+        atom_foreach(c->window, net_wm_state, XA_ATOM,
+            handle_net_wm_state_item, (void *)c);
+    } else if (e->atom == net_wm_desktop) {
+        if(get_atom(c->window, net_wm_desktop, XA_CARDINAL, &c->desktop))
+            desk_update(c);
     }
 }
 
