@@ -1,40 +1,56 @@
 /* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- *
- * Copyright 1998-2006 Decklin Foster <decklin@red-bean.com>. This
- * program is free software; please see LICENSE for details. */
+ * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
+ * This program is free software; see LICENSE for details. */
 
 #include <signal.h>
 #include <X11/cursorfont.h>
 #include "aewm.h"
+#include "atom.h"
 #include "parser.h"
 
 Display *dpy;
 Window root;
 int screen;
+
+client_t *head_client;
+long cur_desk;
+
 XFontStruct *font;
 #ifdef XFT
 XftFont *xftfont;
 XftColor xft_fg;
 #endif
+
 GC invert_gc;
 GC string_gc;
 GC border_gc;
 XColor fg;
 XColor bg;
 XColor bd;
+
 Cursor move_curs;
 Cursor resize_curs;
+
 Atom wm_state;
 Atom wm_change_state;
 Atom wm_protos;
 Atom wm_delete;
-#ifdef GNOME_PDA
-Atom gnome_pda;
-#endif
 #ifdef MWM_HINTS
 Atom mwm_hints;
 #endif
-client_t *head_client;
+Atom net_current_desktop;
+Atom net_num_desktops;
+Atom net_client_list;
+Atom net_wm_desktop;
+Atom net_wm_state;
+Atom net_wm_state_shaded;
+Atom net_wm_state_max_vert;
+Atom net_wm_state_max_horz;
+Atom net_wm_strut;
+Atom net_wm_strut_partial;
+Atom net_wm_window_type;
+Atom net_wm_window_type_dock;
+
 static char *opt_font = DEF_FONT;
 static char *opt_fg = DEF_FG;
 static char *opt_bg = DEF_BG;
@@ -44,6 +60,8 @@ char *opt_new2 = DEF_NEW2;
 char *opt_new3 = DEF_NEW3;
 int opt_bw = DEF_BW;
 int opt_pad = DEF_PAD;
+int opt_ndesks = DEF_DESKS;
+
 #ifdef SHAPE
 Bool shape;
 int shape_event;
@@ -62,6 +80,7 @@ static void read_config(char *);
         "            [--bdcolor|-bd <color>]\n" \
         "            [--bdwidth|-bw <integer>]\n" \
         "            [--padding|-p <integer>]\n" \
+        "            [--ndesks|-n <integer>]\n" \
         "            [--new1|-1 <command>]\n" \
         "            [--new2|-2 <command>]\n" \
         "            [--new3|-3 <command>]\n" \
@@ -85,23 +104,27 @@ int main(int argc, char **argv)
             continue;
         }
         if ARG("fgcolor", "fg", 1) {
-            opt_fg   = argv[++i];
+            opt_fg = argv[++i];
             continue;
         }
         if ARG("bgcolor", "bg", 1) {
-            opt_bg   = argv[++i];
+            opt_bg = argv[++i];
             continue;
         }
         if ARG("bdcolor", "bd", 1) {
-            opt_bd   = argv[++i];
+            opt_bd = argv[++i];
             continue;
         }
         if ARG("bdwidth", "bw", 1) {
-            opt_bw   = atoi(argv[++i]);
+            opt_bw = atoi(argv[++i]);
             continue;
         }
         if ARG("padding", "p", 1) {
-            opt_pad  = atoi(argv[++i]);
+            opt_pad = atoi(argv[++i]);
+            continue;
+        }
+        if ARG("ndesks", "n", 1) {
+            opt_ndesks = atoi(argv[++i]);
             continue;
         }
         if ARG("new1", "1", 1) {
@@ -178,23 +201,55 @@ static void setup_display(void)
     screen = DefaultScreen(dpy);
     root = RootWindow(dpy, screen);
 
-    wm_protos = XInternAtom(dpy, "WM_PROTOCOLS", False);
-    wm_delete = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
-    wm_state = XInternAtom(dpy, "WM_STATE", False);
-    wm_change_state = XInternAtom(dpy, "WM_CHANGE_STATE", False);
-#ifdef GNOME_PDA
-    gnome_pda = XInternAtom(dpy, "GNOME_PANEL_DESKTOP_AREA", False);
-#endif
+    wm_protos = XInternAtom(dpy,
+        "WM_PROTOCOLS", False);
+    wm_delete = XInternAtom(dpy,
+        "WM_DELETE_WINDOW", False);
+    wm_state = XInternAtom(dpy,
+        "WM_STATE", False);
+    wm_change_state = XInternAtom(dpy,
+        "WM_CHANGE_STATE", False);
 #ifdef MWM_HINTS
-    mwm_hints = XInternAtom(dpy, _XA_MWM_HINTS, False);
+    mwm_hints = XInternAtom(dpy,
+        _XA_MWM_HINTS, False);
 #endif
+    net_current_desktop = XInternAtom(dpy,
+        "_NET_CURRENT_DESKTOP", False);
+    net_num_desktops = XInternAtom(dpy,
+        "_NET_NUMBER_OF_DESKTOPS", False);
+    net_wm_desktop = XInternAtom(dpy,
+        "_NET_WM_DESKTOP", False);
+    net_client_list = XInternAtom(dpy,
+        "_NET_CLIENT_LIST", False);
+    net_wm_state = XInternAtom(dpy,
+        "_NET_WM_STATE", False);
+    net_wm_state_shaded = XInternAtom(dpy,
+        "_NET_WM_STATE_SHADED", False);
+    net_wm_state_max_vert = XInternAtom(dpy,
+        "_NET_WM_STATE_MAXIMIZED_VERT", False);
+    net_wm_state_max_horz = XInternAtom(dpy,
+        "_NET_WM_STATE_MAXIMIZED_HORZ", False);
+    net_wm_strut = XInternAtom(dpy,
+        "_NET_WM_STRUT", False);
+    net_wm_strut_partial = XInternAtom(dpy,
+        "_NET_WM_STRUT_PARTIAL", False);
+    net_wm_window_type = XInternAtom(dpy,
+        "_NET_WM_WINDOW_TYPE", False);
+    net_wm_window_type_dock = XInternAtom(dpy,
+        "_NET_WM_WINDOW_TYPE_DOCK", False);
 
-    XAllocNamedColor(dpy, DefaultColormap(dpy, screen), opt_fg, &fg, &dummyc);
-    XAllocNamedColor(dpy, DefaultColormap(dpy, screen), opt_bg, &bg, &dummyc);
-    XAllocNamedColor(dpy, DefaultColormap(dpy, screen), opt_bd, &bd, &dummyc);
+    XAllocNamedColor(dpy, DefaultColormap(dpy, screen),
+        opt_fg, &fg, &dummyc);
+    XAllocNamedColor(dpy, DefaultColormap(dpy, screen),
+        opt_bg, &bg, &dummyc);
+    XAllocNamedColor(dpy, DefaultColormap(dpy, screen),
+        opt_bd, &bd, &dummyc);
 
     font = XLoadQueryFont(dpy, opt_font);
-    if (!font) { err("font '%s' not found", opt_font); exit(1); }
+    if (!font) {
+        err("font '%s' not found", opt_font);
+        exit(1);
+    }
 
 #ifdef XFT
     xft_fg.color.red = fg.red;
@@ -204,7 +259,10 @@ static void setup_display(void)
     xft_fg.pixel = fg.pixel;
 
     xftfont = XftFontOpenXlfd(dpy, DefaultScreen(dpy), opt_font);
-    if (!xftfont) { err("font '%s' not found", opt_font); exit(1); }
+    if (!xftfont) {
+        err("font '%s' not found", opt_font);
+        exit(1);
+    }
 #endif
 
 #ifdef SHAPE
@@ -217,18 +275,29 @@ static void setup_display(void)
     gv.function = GXcopy;
     gv.foreground = fg.pixel;
     gv.font = font->fid;
-    string_gc = XCreateGC(dpy, root, GCFunction|GCForeground|GCFont, &gv);
+    string_gc = XCreateGC(dpy, root,
+        GCFunction|GCForeground|GCFont, &gv);
 
     gv.foreground = bd.pixel;
     gv.line_width = opt_bw;
-    border_gc = XCreateGC(dpy, root, GCFunction|GCForeground|GCLineWidth, &gv);
+    border_gc = XCreateGC(dpy, root,
+        GCFunction|GCForeground|GCLineWidth, &gv);
 
     gv.function = GXinvert;
     gv.subwindow_mode = IncludeInferiors;
-    invert_gc = XCreateGC(dpy, root, GCFunction|GCSubwindowMode|GCLineWidth|GCFont, &gv);
+    invert_gc = XCreateGC(dpy, root,
+        GCFunction|GCSubwindowMode|GCLineWidth|GCFont, &gv);
 
     sattr.event_mask = ChildMask|ColormapChangeMask|ButtonMask;
     XChangeWindowAttributes(dpy, root, CWEventMask, &sattr);
+
+    if (get_atom(root, net_current_desktop, XA_CARDINAL, &cur_desk)) {
+        if (cur_desk >= opt_ndesks)
+            cur_desk = opt_ndesks - 1;
+    } else {
+        cur_desk = 0;
+    }
+    set_atom(root, net_num_desktops, XA_CARDINAL, opt_ndesks);
 }
 
 static void read_config_helper(FILE *rc)
@@ -259,6 +328,9 @@ static void read_config_helper(FILE *rc)
             } else if (strcmp(token, "padding") == 0) {
                 if (get_token(&p, token))
                     opt_pad = atoi(token);
+            } else if (strcmp(token, "ndesks") == 0) {
+                if (get_token(&p, token))
+                    opt_ndesks = atoi(token);
             } else if (strcmp(token, "button1") == 0) {
                 if (get_token(&p, token))
                     opt_new1 = strdup(token);
