@@ -1,9 +1,9 @@
 /* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- *
- * Copyright 1998-2006 Decklin Foster <decklin@red-bean.com>. This
- * program is free software; please see LICENSE for details. */
+ * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
+ * This program is free software; see LICENSE for details. */
 
 #include "aewm.h"
+#include "atom.h"
 
 static void drag(client_t *);
 static void sweep(client_t *);
@@ -11,24 +11,49 @@ static void recalc_sweep(client_t *, int, int, int, int);
 static void draw_outline(client_t *);
 static int get_incsize(client_t *, int *, int *, int);
 
+void switch_desk(int new_desk)
+{
+    client_t *c;
+
+    cur_desk = new_desk;
+    set_atom(root, net_current_desktop, XA_CARDINAL, cur_desk);
+
+    for (c = head_client; c; c = c->next)
+        desk_update(c);
+}
+
+void desk_update(client_t *c)
+{
+    if (ON_CUR_DESK(c)) {
+        if (get_wm_state(c) == NormalState)
+            XMapWindow(dpy, c->frame);
+    } else {
+        XUnmapWindow(dpy, c->frame);
+    }
+}
+
 void move(client_t *c)
 {
-    drag(c);
-    XMoveWindow(dpy, c->frame, c->x, c->y - theight(c));
-    send_config(c);
+    if (!c->max_vert && !c->max_horz) {
+        drag(c);
+        XMoveWindow(dpy, c->frame, c->x, c->y - theight(c));
+        send_config(c);
+    }
 }
 
 void resize(client_t *c)
 {
-    sweep(c);
-    XMoveResizeWindow(dpy, c->frame,
-        c->x, c->y - theight(c), c->width, c->height + theight(c));
-    XMoveResizeWindow(dpy, c->window,
-        0, theight(c), c->width, c->height);
-    send_config(c);
+    if (!c->max_vert && !c->max_horz) {
+        sweep(c);
+        XMoveResizeWindow(dpy, c->frame,
+            c->x, c->y - theight(c), c->width, c->height + theight(c));
+        XMoveResizeWindow(dpy, c->window,
+            0, theight(c), c->width, c->height);
+        send_config(c);
+    }
 }
 
-void hide(client_t *c)
+void iconify_win(client_t *c)
 {
     if (!c->ignore_unmap) c->ignore_unmap++;
     XUnmapWindow(dpy, c->frame);
@@ -36,6 +61,102 @@ void hide(client_t *c)
     set_wm_state(c, IconicState);
 }
 
+void shade_win(client_t *c)
+{
+    if (!c->shaded) {
+        c->shaded = 1;
+        XMoveResizeWindow(dpy, c->frame,
+            c->x, c->y - theight(c),
+            c->width, theight(c) - BW(c));
+        append_to_atom(c->window, net_wm_state, XA_ATOM,
+            net_wm_state_shaded);
+    } else {
+        c->shaded = 0;
+        XMoveResizeWindow(dpy, c->frame,
+            c->x, c->y - theight(c),
+            c->width, c->height + theight(c));
+        remove_from_atom(c->window, net_wm_state, XA_ATOM,
+            net_wm_state_shaded);
+    }
+
+    send_config(c);
+}
+
+/* There are two state variables here, because the EWMH defines both
+ * horizontal and vertical maximization. However, we only do
+ * both-at-once maximization. Support for all 3 possibilites could be
+ * added, but there's no UI for it.
+ *
+ * Totally arbitrary decision about the semantics of maximization: if
+ * you maximize something, that unshades it. I figure this is what I
+ * would want 99% of the time anyway. Unmaximizing, however, does not
+ * unshade (same rationale). Feel free to offer better ideas if you
+ * have them, though.
+ *
+ * About save_x, etc: they are currently assumed to be meaningful iff
+ * c->max_vert && c->max_horz. If you change this, caveat hacker. They
+ * also seem kind of kludgy to me and may go away soon. */
+
+void maximize_win(client_t *c)
+{
+    strut_t s;
+
+    if (!c->max_vert || !c->max_horz) {
+        accumulate_struts(c->desktop, &s);
+        c->shaded = 0;
+        c->max_vert = 1;
+        c->max_horz = 1;
+        remove_from_atom(c->window, net_wm_state, XA_ATOM,
+            net_wm_state_shaded);
+        append_to_atom(c->window, net_wm_state, XA_ATOM,
+            net_wm_state_max_vert);
+        append_to_atom(c->window, net_wm_state, XA_ATOM,
+            net_wm_state_max_horz);
+        c->save_x = c->x;
+        c->save_y = c->y;
+        c->save_width = c->width;
+        c->save_height = c->height;
+        c->x = s.left;
+        c->y = s.top + theight(c);
+        c->width = DisplayWidth(dpy, screen) - 2*BW(c) -
+            s.left - s.right;
+        c->height = DisplayHeight(dpy, screen) - 2*BW(c) - theight(c) -
+            s.top - s.bottom;
+        XMoveResizeWindow(dpy, c->frame, c->x, c->y - theight(c),
+            c->width, c->height + theight(c));
+        XResizeWindow(dpy, c->window, c->width, c->height);
+    } else {
+        c->max_vert = 0;
+        c->max_horz = 0;
+        remove_from_atom(c->window, net_wm_state, XA_ATOM,
+            net_wm_state_max_vert);
+        remove_from_atom(c->window, net_wm_state, XA_ATOM,
+            net_wm_state_max_horz);
+        c->x = c->save_x;
+        c->y = c->save_y;
+        c->width = c->save_width;
+        c->height = c->save_height;
+        XMoveResizeWindow(dpy, c->frame, c->x, c->y - theight(c),
+            c->width, c->shaded ? theight(c) - BW(c) : c->height + theight(c));
+        XResizeWindow(dpy, c->window, c->width, c->height);
+    }
+
+    send_config(c);
+}
+
+void toggle_all_desks(client_t *c)
+{
+    if (c->desktop == DESK_ALL) {
+        c->desktop = cur_desk;
+        set_atom(c->window, net_wm_desktop, XA_CARDINAL, cur_desk);
+    } else {
+        c->desktop = DESK_ALL;
+        set_atom(c->window, net_wm_desktop, XA_CARDINAL, DESK_ALL);
+    }
+
+    redraw(c);
+}
+
 /* The name of this function is a bit misleading: if the client
  * doesn't listen to WM_DELETE then we just terminate it with extreme
  * prejudice. */
@@ -183,7 +304,7 @@ static int get_incsize(client_t *c, int *x_ret, int *y_ret, int mode)
         if (mode == SIZE_PIXELS) {
             *x_ret = c->width - ((c->width - base_width) % width_inc);
             *y_ret = c->height - ((c->height - base_height) % height_inc);
-        } else if (mode == SIZE_LOGICAL) {
+        } else /* mode == SIZE_LOGICAL */ {
             *x_ret = (c->width - base_width) / width_inc;
             *y_ret = (c->height - base_height) / height_inc;
         }
