@@ -1,13 +1,14 @@
 /* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- *
- * Copyright 1998-2006 Decklin Foster <decklin@red-bean.com>. This
- * program is free software; please see LICENSE for details. */
+ * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
+ * This program is free software; see LICENSE for details. */
 
 #include <stdarg.h>
 #include <unistd.h>
 #include <signal.h>
 #include <sys/wait.h>
 #include "aewm.h"
+#include "atom.h"
+#include "strut.h"
 
 static void quit_nicely(void);
 
@@ -100,6 +101,91 @@ void get_mouse_position(int *x, int *y)
         x, y, &win_x, &win_y, &mask);
 }
 
+/* I would really prefer to implement this using atom_foreach, but it
+ * would be just as hairy since we don't know how much memory to
+ * allocate at the start, etc. I really want to find whoever designed
+ * this particular interface and smack them upside the head. */
+
+void remove_from_atom(Window w, Atom a, Atom type, long remove_val)
+{
+    Atom real_type;
+    int real_format = 0;
+    unsigned long items_read = 0;
+    unsigned long bytes_left = 0;
+    unsigned char *data;
+    long start = 0;
+    size_t vals_alloc = 10;
+    size_t nvals = 0;
+    long *values = malloc(vals_alloc * sizeof *values);
+
+    do {
+        XGetWindowProperty(dpy, w, a, start, 1, False, type,
+            &real_type, &real_format,
+            &items_read, &bytes_left, &data);
+        if (real_format == 32 && items_read >= 1) {
+            if (*(long *)data != remove_val) {
+                if (nvals == vals_alloc) {
+                    vals_alloc *= 2;
+                    values = realloc(values,
+                        vals_alloc * sizeof *values);
+                }
+                values[nvals++] = *(long *)data;
+            }
+            XFree(data);
+            start += items_read;
+        }
+    } while (real_format == 32 && bytes_left);
+
+    XChangeProperty(dpy, w, a, type, 32,
+        PropModeReplace, (unsigned char *)values, nvals);
+    free(values);
+}
+
+/* We use XQueryTree instead of walking the clients list for two
+ * reasons: first, we may be in the process of mapping a new window,
+ * in which case its client structure has already been inserted into
+ * the list. We don't want to consider the strut on the window we're
+ * mapping! Second, we may be starting in the middle of an X session,
+ * and reparenting a maximized window before a window with a strut
+ * set. We check if the window is mapped already; however, we still
+ * have to call find_client in case we have reparented the window
+ * and our frame is the direct child of root. */
+
+void accumulate_struts(int desk, strut_t *s)
+{
+    unsigned int nwins, i;
+    Window dummyw1, dummyw2, *wins, w;
+    long w_desk = cur_desk;
+    XWindowAttributes attr;
+    client_t *c;
+    strut_t temp;
+
+    s->left = 0;
+    s->right = 0;
+    s->top = 0;
+    s->bottom = 0;
+
+    XQueryTree(dpy, root, &dummyw1, &dummyw2, &wins, &nwins);
+    for (i = 0; i < nwins; i++) {
+        c = find_client(wins[i], MATCH_FRAME);
+        w = c ? c->window : wins[i];
+        get_atom(w, net_wm_desktop, XA_CARDINAL, &w_desk);
+        XGetWindowAttributes(dpy, w, &attr);
+        if (attr.map_state == IsViewable && (w_desk == desk ||
+                w_desk == DESK_ALL) && get_strut(w, &temp)) {
+            if (temp.left > s->left)
+                s->left = temp.left;
+            if (temp.right > s->right)
+                s->right = temp.right;
+            if (temp.top > s->top)
+                s->top = temp.top;
+            if (temp.bottom > s->bottom)
+                s->bottom = temp.bottom;
+        }
+    }
+    XFree(wins);
+}
+
 #ifdef DEBUG
 
 /* Bleh, stupid macro names. I'm not feeling creative today. */
@@ -135,12 +221,12 @@ void show_event(XEvent e)
         SHOW_EV(ResizeRequest, xresizerequest)
         SHOW_EV(UnmapNotify, xunmap)
         default:
-        if (shape && e.type == shape_event) {
-            ev_type = "ShapeNotify"; w = ((XShapeEvent *)&e)->window;
-        } else {
-            ev_type = "unknown event"; w = None;
-        }
-        break;
+            if (shape && e.type == shape_event) {
+                ev_type = "ShapeNotify"; w = ((XShapeEvent *)&e)->window;
+            } else {
+                ev_type = "unknown event"; w = None;
+            }
+            break;
     }
 
     if ((c = find_client(w, MATCH_WINDOW))) {
@@ -256,6 +342,8 @@ static void quit_nicely(void)
     XInstallColormap(dpy, DefaultColormap(dpy, screen));
     XSetInputFocus(dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
 
+    XDeleteProperty(dpy, root, net_client_list);
+
     XCloseDisplay(dpy);
     exit(0);
 }
