@@ -1,17 +1,14 @@
 /* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- *
- * Copyright 1998-2006 Decklin Foster <decklin@red-bean.com>. This
- * program is free software; please see LICENSE for details. */
+ * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
+ * This program is free software; see LICENSE for details. */
 
 #include "aewm.h"
-#ifdef GNOME_PDA
-#include "gnome-pda.h"
-#endif
+#include "atom.h"
 
-static void init_position(client_t *);
-static void reparent(client_t *);
+static void init_position(client_t *, strut_t *);
+static void reparent(client_t *, strut_t *);
 #ifdef MWM_HINTS
-static PropMwmHints *get_mwm_hints(Window);
+static int get_mwm_hints(Window, PropMwmHints *);
 #endif
 
 /* Set up a client structure for the new (not-yet-mapped) window. The
@@ -27,8 +24,10 @@ void make_new_client(Window w)
     XWindowAttributes attr;
     XWMHints *hints;
 #ifdef MWM_HINTS
-    PropMwmHints *mhints;
+    PropMwmHints mhints;
 #endif
+    Atom win_type;
+    strut_t s;
     long dummy;
 
     c = malloc(sizeof *c);
@@ -41,6 +40,7 @@ void make_new_client(Window w)
     XFetchName(dpy, w, &c->name);
     XGetWindowAttributes(dpy, w, &attr);
 
+    c->frame = None;
     c->window = w;
     c->ignore_unmap = 0;
     c->x = attr.x;
@@ -48,6 +48,12 @@ void make_new_client(Window w)
     c->width = attr.width;
     c->height = attr.height;
     c->cmap = attr.colormap;
+    c->has_title = 1;
+    c->has_border = 1;
+    c->shaded = 0;
+    c->max_vert = 0;
+    c->max_horz = 0;
+
     c->size = XAllocSizeHints();
     XGetWMNormalHints(dpy, c->window, c->size, &dummy);
 
@@ -65,23 +71,39 @@ void make_new_client(Window w)
 #endif
 
 #ifdef MWM_HINTS
-    c->has_title = 1;
-    c->has_border = 1;
-
-    if ((mhints = get_mwm_hints(c->window))) {
-        if (mhints->flags & MWM_HINTS_DECORATIONS
-            && !(mhints->decorations & MWM_DECOR_ALL)) {
-            c->has_title  = mhints->decorations & MWM_DECOR_TITLE;
-            c->has_border = mhints->decorations & MWM_DECOR_BORDER;
+    if (get_mwm_hints(c->window, &mhints)) {
+        if (mhints.flags & MWM_HINTS_DECORATIONS
+            && !(mhints.decorations & MWM_DECOR_ALL)) {
+            c->has_title  = mhints.decorations & MWM_DECOR_TITLE;
+            c->has_border = mhints.decorations & MWM_DECOR_BORDER;
         }
-        XFree(mhints);
     }
 #endif
 
+    if (get_atom(c->window, net_wm_window_type, XA_ATOM, &win_type)) {
+        if (win_type == net_wm_window_type_dock) {
+            c->has_title = 0;
+            c->has_border = 0;
+        }
+    }
+
+    atom_foreach(c->window, net_wm_state, XA_ATOM,
+        handle_net_wm_state_item, (void *)c);
+
+    if (get_atom(c->window, net_wm_desktop, XA_CARDINAL, &c->desktop)) {
+        if (c->desktop >= opt_ndesks)
+            c->desktop = opt_ndesks - 1;
+    } else {
+        set_atom(c->window, net_wm_desktop, XA_CARDINAL, cur_desk);
+        c->desktop = cur_desk;
+    }
+
+    accumulate_struts(c->desktop, &s);
+
     if (attr.map_state == IsViewable) {
         c->ignore_unmap++;
     } else {
-        init_position(c);
+        init_position(c, &s);
         set_wm_state(c, NormalState);
         if ((hints = XGetWMHints(dpy, w))) {
             if (hints->flags & StateHint) set_wm_state(c, hints->initial_state);
@@ -95,7 +117,7 @@ void make_new_client(Window w)
 #endif
 
     gravitate(c, GRAV_APPLY);
-    reparent(c);
+    reparent(c, &s);
 
 #ifdef XFT
     c->xftdraw = XftDrawCreate(dpy, (Drawable)c->frame,
@@ -109,37 +131,42 @@ void make_new_client(Window w)
             XUnmapWindow(dpy, c->window);
         } else {
             XMapWindow(dpy, c->window);
-            XMapRaised(dpy, c->frame);
+            if (ON_CUR_DESK(c)) XMapRaised(dpy, c->frame);
             set_wm_state(c, NormalState);
         }
     } else {
         if (get_wm_state(c) == NormalState) {
             XMapWindow(dpy, c->window);
-            XMapRaised(dpy, c->frame);
+            if (ON_CUR_DESK(c)) XMapRaised(dpy, c->frame);
         }
     }
 
+    append_to_atom(root, net_client_list, XA_WINDOW, c->window);
+
     XSync(dpy, False);
     XUngrabServer(dpy);
 }
 
-/* This one does -not- free the data coming back from Xlib; it just
- * sends back the pointer to what was allocated. */
-
 #ifdef MWM_HINTS
-static PropMwmHints *get_mwm_hints(Window w)
+static int get_mwm_hints(Window w, PropMwmHints *h)
 {
-    Atom real_type; int real_format;
-    unsigned long items_read, bytes_left;
+    Atom real_type;
+    int real_format =0;
+    unsigned long items_read = 0;
+    unsigned long bytes_left = 0;
     unsigned char *data;
 
-    if (XGetWindowProperty(dpy, w, mwm_hints, 0L, 20L, False,
-            mwm_hints, &real_type, &real_format, &items_read, &bytes_left,
-            &data) == Success
-        && items_read >= PROP_MOTIF_WM_HINTS_ELEMENTS) {
-        return (PropMwmHints *)data;
+    XGetWindowProperty(dpy, w, mwm_hints,
+        0, PROP_MOTIF_WM_HINTS_ELEMENTS, False, mwm_hints,
+        &real_type, &real_format, &items_read, &bytes_left, &data);
+
+    if (real_format == 32 && items_read >= PROP_MOTIF_WM_HINTS_ELEMENTS) {
+        /* Look ma, no cast! */
+        memcpy(h, data, sizeof *h);
+        XFree(data);
+        return 1;
     } else {
-        return NULL;
+        return 0;
     }
 }
 #endif
@@ -159,21 +186,15 @@ static PropMwmHints *get_mwm_hints(Window w)
  * we add theight into the calculation and then degravitate. Don't
  * think about it too hard, or your head will explode.
  *
- * If we are using the gnome_pda hint, the entire process is done over
- * a smaller "pretend" root window, and then at the very end we shift
- * the window into the right place based using the left/top offsets. */
+ * We also subtract any struts (accumulated in s) from the range of x
+ * and y values, and offset by the top/left of the strut, since we
+ * don't want to map over a strut. */
 
-static void init_position(client_t *c)
+static void init_position(client_t *c, strut_t *s)
 {
-    int xmax = DisplayWidth(dpy, screen);
-    int ymax = DisplayHeight(dpy, screen);
-
-#ifdef GNOME_PDA
-    deskarea_t da;
-    get_gnome_pda(&da);
-    xmax -= da.left + da.right;
-    ymax -= da.top + da.bottom;
-#endif
+    int xmax = DisplayWidth(dpy, screen) - s->left - s->right;
+    int ymax = DisplayHeight(dpy, screen) - s->top - s->bottom;
+    Atom win_type;
 
     if (c->size->flags & (USSize)) {
         if (c->size->width) c->width = c->size->width;
@@ -186,56 +207,79 @@ static void init_position(client_t *c)
     }
 
     if (c->size->flags & USPosition) {
-        /* always obey */
         c->x = c->size->x;
         c->y = c->size->y;
-    } else if (c->size->flags & PPosition) {
-        /* only if it's sane. I'm looking at you, Netscape, Emacs, GTK... */
-        if (c->size->x != 0 && c->size->y != 0) {
+    } else {
+        if (c->size->flags & PPosition) {
             c->x = c->size->x;
             c->y = c->size->y;
         }
+        if (c->x < 0) c->x = 0;
+        if (c->y < 0) c->y = 0;
+        if (c->x > xmax) c->x = xmax - theight(c);
+        if (c->y > ymax) c->y = ymax - theight(c);
+
+        /* If it's -not- a dock, but it is at 0,0, we want to place it.
+         * Docks are the the only types of windows that we let get
+         * away with placing themselves at 0,0. */
+
+        if (!(get_atom(c->window, net_wm_window_type, XA_ATOM, &win_type) &&
+                win_type == net_wm_window_type_dock) &&
+                c->x == 0 && c->y == 0) {
+            int mouse_x, mouse_y;
+            get_mouse_position(&mouse_x, &mouse_y);
+            if (c->width < xmax)
+                c->x = (mouse_x < xmax ? (mouse_x / (float)xmax) : 1)
+                    * (xmax - c->width - 2*BW(c));
+            if (c->height + theight(c) < ymax)
+                c->y = (mouse_y < ymax ? (mouse_y / (float)ymax) : 1)
+                    * (ymax - c->height - theight(c) - 2*BW(c));
+            c->y += theight(c);
+            gravitate(c, GRAV_UNDO);
+        }
     }
 
-    /* Even if we got a hint for x and y, we have to check that the
-     * client isn't being insane. If it is, just reset things to 0 so we
-     * can place it ourselves. */
-
-    if (c->x < 0 || c->x > xmax) c->x = 0;
-    if (c->y < 0 || c->y > ymax) c->y = 0;
-
-    /* If it's still at 0,0, we want to place it. */
-    if (c->x == 0 && c->y == 0) {
-        int mouse_x, mouse_y;
-        get_mouse_position(&mouse_x, &mouse_y);
-        if (c->width < xmax)
-            c->x = (mouse_x < xmax ? (mouse_x / (float)xmax) : 1)
-                * (xmax - c->width - 2*BW(c));
-        if (c->height + theight(c) < ymax)
-            c->y = (mouse_y < ymax ? (mouse_y / (float)ymax) : 1)
-                * (ymax - c->height - theight(c) - 2*BW(c));
-        c->y += theight(c);
-        gravitate(c, GRAV_UNDO);
-    }
-
-#ifdef GNOME_PDA
-    c->x += da.left;
-    c->y += da.top;
-#endif
+    c->x += s->left;
+    c->y += s->top;
 }
 
 /* Stick the client's window into our frame. */
 
-static void reparent(client_t *c)
+static void reparent(client_t *c, strut_t *s)
 {
     XSetWindowAttributes pattr;
+    int fx, fy, fw, fh;
+
+    if (c->max_vert && c->max_horz) {
+        c->save_x = c->x;
+        c->save_y = c->y;
+        c->save_width = c->width;
+        c->save_height = c->height;
+        c->x = s->left;
+        c->y = s->top + theight(c);
+        c->width = DisplayWidth(dpy, screen) - 2*BW(c) -
+            s->left - s->right;
+        c->height = DisplayHeight(dpy, screen) - 2*BW(c) - theight(c) -
+            s->top - s->bottom;
+    }
+
+    if (c->shaded) {
+        fx = c->x;
+        fy = c->y - theight(c);
+        fw = c->width;
+        fh = theight(c) - BW(c);
+    } else {
+        fx = c->x;
+        fy = c->y - theight(c);
+        fw = c->width;
+        fh = c->height + theight(c);
+    }
 
     pattr.override_redirect = True;
     pattr.background_pixel = bg.pixel;
     pattr.border_pixel = bd.pixel;
     pattr.event_mask = ChildMask|ButtonPressMask|ExposureMask|EnterWindowMask;
-    c->frame = XCreateWindow(dpy, root,
-        c->x, c->y - theight(c), c->width, c->height + theight(c), BW(c),
+    c->frame = XCreateWindow(dpy, root, fx, fy, fw, fh, BW(c),
         DefaultDepth(dpy, screen), CopyFromParent, DefaultVisual(dpy, screen),
         CWOverrideRedirect|CWBackPixel|CWBorderPixel|CWEventMask, &pattr);
 
